library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity Chrono2 is
    Port (
            clk             : in std_logic;
            reset           : in std_logic;

            -- Outputs (Contadores BCD)
            segundos        : out std_logic_vector(3 downto 0);
            dec_segundos    : out std_logic_vector(3 downto 0);
            minutos         : out std_logic_vector(3 downto 0);
            dec_minutos     : out std_logic_vector(3 downto 0));
end Chrono2;

architecture Behavioral of Chrono2 is
  
    ----------------------
    --seales divisor
    ----------------------
    
        constant MaxCount               : integer := 125000000;
        signal Count                    : integer range 0 to MaxCount-1;
    
     ----------------------
     --contantes contadores
     ----------------------    
        constant MaxCountDecenasSeg    : unsigned(3 downto 0) :="0110";
        constant MaxCountUnidadesSeg   : unsigned(3 downto 0) :="1010";
        constant MaxCountDecenasMin    : unsigned(3 downto 0) :="0110";
        constant MaxCountUnidadesMin   : unsigned(3 downto 0) :="1010";
    
     ----------------------
     --seales contador
     ----------------------
   
        signal CountDecenasSeg         : unsigned(3 downto 0);
        signal CountUnidadesSeg        : unsigned(3 downto 0);
        signal CountDecenasMin         : unsigned(3 downto 0);
        signal CountUnidadesMin        : unsigned(3 downto 0);
        
    ----------------------
     --seales paso contadores 
     ----------------------    
 
        signal Enable             : std_logic;
        signal Enable_segundos    : std_logic;
        signal Enable_dec_segundos: std_logic;
        signal Enable_dec_minutos : std_logic;
        signal Enable_minutos     : std_logic;
        signal Enable_contador    : std_logic;
        signal Enable_cambio      :std_logic;
        signal Enable_refresco    :std_logic;

    ----------------------------
    ----seleccion
    ----------------------------
    signal seleccionar        :std_logic;


    
begin
    Enable<='1';
    
    --Dividor de frecuencia
    process(clk,reset)
    begin
    if(reset ='1') then
        Count <=0;
    elsif(clk'event and clk ='1') then
        if(Enable='1') then
            if(Count =MaxCount-1) then
                Count <= 0;
            else
                Count <= Count+1;
            end if;
         end if;
     end if;
     end process;

     Enable_Segundos <= '1' when Count =MaxCount-1 else '0';


--------------------------------------------------------------------------------------------------

---contador de segundos: segundos de 0 a 9

---------------------------------------------------------------------------------

    process(clk,reset)
         begin 
             if(reset='1') then
                  CountUnidadesSeg <= "0000";  
             elsif (clk'event and clk='1') then
                   if (Enable_segundos ='1') then
                        if(CountUnidadesSeg = MaxCountUnidadesSeg-1)  then --si lo que cuento es 10-1=9 entonces lo pongo a cero
                             CountUnidadesSeg <="0000";
                        else    
                              CountUnidadesSeg <= CountUnidadesSeg +1; --mientras no llege a nueve voy sumando   
                        end if;
                   end if;
             end if;
    end process;

    ------habilito la seal cuando el contador llegue a 9 y los sea el flanco de subida del divisor 
    ------ porque se mantendra encendido durante 125MHz , tienen que coicidir ambos
    
 Enable_dec_segundos <= '1' when ((CountUnidadesSeg=(MaxCountUnidadesSeg-1)) and (Enable_segundos='1' ))else '0';
 -- pondre a 1 el enable del contador de decenas de segundo cuando CountUnidadesSeg=9, he llegado a su mximo, pongo 
 -- este ultimo a 0 y ya empiezo a contar decenas de segundo
    
segundos <=  std_logic_vector(CountUnidadesSeg);   


-----------------------------------------------------------------

---contador de decenas de segundos que se incremente cuando el contador de segundos llegue a nueve

-------------------------------------------------
 process(clk,reset)
         begin 
             if(reset='1') then
                  CountDecenasSeg <= "0000";  
             elsif (clk'event and clk='1') then
                   if (Enable_dec_segundos ='1') then
                        if(CountDecenasSeg = MaxCountDecenasSeg-1)  then --si lo que cuento es 6-1=5 entonces lo pongo a cero
                             CountDecenasSeg <="0000";
                        else    
                              CountDecenasSeg <= CountDecenasSeg +1;  --mientras no llege a 5 voy sumando  
                        end if;
                   end if;
             end if;
    end process;
    
    
 Enable_minutos <= '1' when ((CountDecenasSeg=MaxCountDecenasSeg-1) and  (Enable_dec_segundos='1')) else '0';
 -- pondre a 1 el enable del contador de minutos cuando CountDecenasSeg=5 , he llegado a su mximo, pongo este
 -- este ultimo a 0 y ya empiezo a contar decenas de segundo                        
 dec_segundos <= std_logic_vector(CountDecenasSeg);   
                           
----------------------------------------------------------------------

---contador de minutos que se incremente cuando las decenas de segundos llegan a 5 y los segundos a 9

----------------------------------------------------------------------
 process(clk,reset)
         begin 
             if(reset='1') then
                  CountUnidadesMin <= "0000";  
             elsif (clk'event and clk='1') then
                   if (Enable_minutos ='1') then
                        if(CountUnidadesMin = MaxCountUnidadesMin-1)  then
                             CountUnidadesMin <="0000";
                        else    
                              CountUnidadesMin <= CountUnidadesMin +1;            
                        end if;
                   end if;
             end if;
    end process;
    
    
 Enable_dec_minutos <='1' when ((CountUnidadesMin = MaxCountUnidadesMin-1) and (Enable_minutos= '1')) else '0';
     
 minutos <= std_logic_vector(CountUnidadesMin);          
                         
 -------------------------------------------------------------------------
 
----contador decenas de minuto

----------------------------------------------------------------------------
process(clk,reset)
         begin 
             if(reset='1') then
                  CountDecenasMin <= "0000";  
             elsif (clk'event and clk='1') then
                   if (Enable_dec_minutos ='1') then
                        if(CountDecenasMin = CountDecenasMin-1)  then --si lo que cuento es 6-1=5 entonces lo pongo a cero
                             CountDecenasMin <="0000";
                        else    
                              CountDecenasMin <= CountDecenasMin +1;            
                        end if;
                   end if;
             end if;
    end process;
    
   dec_minutos <= std_logic_vector(CountDecenasMin);     



end Behavioral;
